//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Global.hpp"
#include "Object.hpp"
#include "Screen.hpp"
#include "AllObj.hpp"
#include "MyException.hpp"
#include "LuaExpose.hpp"
#include "Console.hpp"
#include "Event.hpp"
#include "SoundManager.hpp"

#include <lua.hpp>
#include <luabind/luabind.hpp>

#include <time.h>
#include <SPK.h>

// Locales
#include <libintl.h>
#include <locale.h>
//------------------------------------------------------------------------------
#include <SFML/Window.hpp>
#include <SFML/Graphics.hpp>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
namespace Global
{
//------------------------------------------------------------------------------
struct globalDataStr
{
    globalDataStr() : currentScreen(NULL), luaState(NULL), console(NULL) {}

    sf::RenderWindow *renderWindow;

    std::vector<Screen*> screenTbl; // The screen are removed from the list when they are destroyed => No need to protect them
    ProtectedPtr<Screen> currentScreen;

    lua_State *luaState;

    ConfigManager confManager;
    Console *console;

    std::string modeToLoad;

    bool drawPhysicObjects;
    bool isRequestRecreateRenderWindow;

    std::vector<std::string> cmdArgTbl;

    // Unique frame id
    int uniqueFrameId;

    // Sounds
    SoundManager *globalSoundManager;
};
//------------------------------------------------------------------------------
globalDataStr *g_global = NULL;
//------------------------------------------------------------------------------
bool g_globalOk = false;
//------------------------------------------------------------------------------
Engine::Engine(int argc, const char* argv[])
{
    // Global data
    g_global = new globalDataStr();

    // Test
    if(g_globalOk == true) // Already created !
    {
        MY_THROW("Engine already created !");
    }

    // Pointers
    g_global->drawPhysicObjects = false;
    g_global->isRequestRecreateRenderWindow = false;
    g_global->renderWindow = NULL;
    g_global->currentScreen = NULL;
    g_global->luaState = NULL;
    g_global->console = NULL;
    g_global->modeToLoad = "";

    // Unique frame id
    g_global->uniqueFrameId = 0;

    // Save agrs list
    g_global->cmdArgTbl.resize(argc);
    for(int i = 0; i < argc; i++)
    {
        g_global->cmdArgTbl[i] = argv[i];
    }

    // Spark random seed
    SPK::randomSeed = static_cast<unsigned int>(time(NULL)); // 1 fois !!

    // Init key event maps
    ButtonEventHandler::FillMaps();

    // Config manager
    g_global->confManager.LoadAll();

    // RenderWindow (needs confManager loaded)
    Priv::CreateRenderWindow();

    // Console (no use if RenderWindow not loaded)
    g_global->console = new Console();

    // Global sound manager
    g_global->globalSoundManager = new SoundManager();

    // GlobalOk
    g_globalOk = true;
}
//------------------------------------------------------------------------------
Engine::~Engine()
{
    /// We delete the screens and the global sound manager but the engine is still ok
    // Screen
    for(std::vector<Screen*>::iterator it = g_global->screenTbl.begin(); it != g_global->screenTbl.end(); it++ )
    {
        delete *it;
    }
    g_global->screenTbl.clear();

    // Sound
    delete g_global->globalSoundManager;

    /// The engine is not ok anymore
    g_globalOk = false;

    // Config manager
    //delete g_global->confManager;

    // Console
    delete g_global->console;

    // Lua
    if(g_global->luaState)
    {
        lua_close(g_global->luaState);
    }

    // App
    if(g_global->renderWindow)
    {
        delete g_global->renderWindow;
    }

    // Destroy global data
    delete g_global;
}
//------------------------------------------------------------------------------
void ShowConsole(bool isVisible)
{
    g_global->console->Show(isVisible);
}
//------------------------------------------------------------------------------
/** Return true if the information console is visible, false if not */
bool IsConsoleVisible()
{
    return g_global->console->IsVisible();
}
//------------------------------------------------------------------------------
/** Get the real pointer to the current screen */
Screen *GetCurrentScreenRaw()
{
    if(!g_global->currentScreen.IsNull())
    {
        return g_global->currentScreen;
    }
    else
    {
        return NULL;
    }
}
//------------------------------------------------------------------------------
Console &GetConsole()
{
    return *(g_global->console);
}
//------------------------------------------------------------------------------
void RequestRecreateRenderWindow()
{
    g_global->isRequestRecreateRenderWindow = true;
}
//------------------------------------------------------------------------------
float GetFrameTime()
{
    return g_global->renderWindow->GetFrameTime();
}
//------------------------------------------------------------------------------
sf::RenderWindow &GetApp()
{
    Priv::CheckEngine();

    return *g_global->renderWindow;
}

//------------------------------------------------------------------------------
// Screen
ProtectedPtr<Screen> CreateScreen(const std::string &name)
{
    return Screen::Create(name);
}
//------------------------------------------------------------------------------
ProtectedPtr<Screen> CreateScreen(const std::string &name, const ScreenConfig &screenConfig)
{
    Screen *screen = CreateScreen(name);
    screen->SetScreenConfig(screenConfig);
    return screen;
}
//------------------------------------------------------------------------------
void AddScreen(Screen *screen)
{
    Priv::CheckEngine();

    g_global->screenTbl.push_back(screen);
    Priv::AddHotKeysEvents(screen);
}
//------------------------------------------------------------------------------
void RemoveScreen(Screen *screen)
{
    Priv::CheckEngine();

    std::vector<Screen*>::iterator it = find(g_global->screenTbl.begin(), g_global->screenTbl.end(), screen);
    if(it != g_global->screenTbl.end())
    {
        g_global->screenTbl.erase(it);
    }
    else
    {
        MY_THROW("RemoveScreen : Pointer not found !");
    }
}
//------------------------------------------------------------------------------
int GetUniqueFrameId()
{
    return g_global->uniqueFrameId;
}
//------------------------------------------------------------------------------
ConfigManager &GetConfigManager()
{
    return g_global->confManager;
}
//------------------------------------------------------------------------------
void ChangeScreen(const std::string &name)
{
    Priv::CheckEngine();

    for(std::vector<Screen*>::iterator it = g_global->screenTbl.begin(); it != g_global->screenTbl.end(); it++ )
    {
        if((*it)->GetName() == name)
        {
            if(!g_global->currentScreen.IsNull())
            {
                g_global->currentScreen->OnLoseFocus();
            }
            g_global->currentScreen = (*it);
            g_global->currentScreen->OnGetFocus();
            return;
        }
    }

    MY_THROW("ChangeScreen : Screen '" << name << "' not found !");
}
//------------------------------------------------------------------------------
void ChangeScreen(ProtectedPtr<Screen> screen)
{
    if(std::find(g_global->screenTbl.begin(), g_global->screenTbl.end(), screen) != g_global->screenTbl.end())
    {
        if(!g_global->currentScreen.IsNull())
        {
            g_global->currentScreen->OnLoseFocus();
        }
        g_global->currentScreen = screen;
        g_global->currentScreen->OnGetFocus();
    }
}
//------------------------------------------------------------------------------
std::string GetEngineVersion()
{
    return VERSION;
}
//------------------------------------------------------------------------------
// Other
void LuaRunScript(const std::string &filename)
{
    Priv::CheckEngine();

    try
    {
        // Now use this class in a lua script
        if(luaL_dofile(g_global->luaState, filename.c_str()) != 0)
        {
            MY_THROW("LUA error :\n" << lua_tostring(g_global->luaState, -1));
        }
    }
    catch(luabind::error &e)
    {
        MY_THROW("LUA error :\n" << lua_tostring(g_global->luaState, -1));
    }
    catch(luabind::cast_failed &e)
    {
        MY_THROW("LUA error :\n" << lua_tostring(g_global->luaState, -1));
    }
}
//------------------------------------------------------------------------------
lua_State *GetLuaState()
{
    Priv::CheckEngine();

    return g_global->luaState;
}
//------------------------------------------------------------------------------
void Quit()
{
    Priv::CheckEngine();

    g_global->renderWindow->Close();
}
//------------------------------------------------------------------------------
void Run()
{
    Priv::CheckEngine();

    if(g_global->renderWindow == NULL)
    {
        MY_THROW("Run : Render window not opened (g_global->RenderWindow is null) !");
    }

    #if DEBUG == 1
    sf::Clock perfClock;
    #endif

    while(g_global->renderWindow->IsOpened())
    {
        #if DEBUG == 1
        perfClock.Reset();
        #endif

        // Load a mode if needed (can only be done in the Run loop)
        if(!g_global->modeToLoad.empty())
        {
            Priv::DoLoadMode();
        }

        // Check the screen
        if(g_global->currentScreen.IsNull())
        {
            MY_THROW("Run : No screen selected (g_global->currentScreen is null) !");
        }

        if(!g_global->currentScreen.IsValid())
        {
            MY_THROW("Run : The Screen is not valid (it was deleted) !");
        }

        // Recreate a render window if needed
        if(g_global->isRequestRecreateRenderWindow)
        {
            Priv::CreateRenderWindow();
            g_global->currentScreen.GetPtr()->OnGetFocus(); // Config the render window for the screen (hide cursor...)
            g_global->isRequestRecreateRenderWindow = false;

            // We render a frame to prevente huge frame time
            g_global->renderWindow->Display();
        }

        // Unique frame id
        g_global->uniqueFrameId++;

        // Run the current screen (events, mouvement...)
        g_global->currentScreen.GetPtr()->Run();

        // Clear the render window
        g_global->renderWindow->Clear(g_global->currentScreen.GetPtr()->GetBackgroundColor());

        // Draw the current screen (g_global->currentScreen may have changed)
        g_global->currentScreen.GetPtr()->Render(*g_global->renderWindow);

        // Draw the console
        g_global->console->Render(*g_global->renderWindow);

        #if DEBUG == 1
        //cout << "Time : " << perfClock.GetElapsedTime() << " s" << endl;
        #endif

        // Display window on screen
        g_global->renderWindow->Display();
    }
}
//------------------------------------------------------------------------------
// Echo
/** Print a message to the engine's console, and to standard output (stdout) */
int printConsole(lua_State* L)
{
    return Priv::printParameters(L, cout);
}
//------------------------------------------------------------------------------
/** Print an error message to the engine's console, and to standard error output (stderr) */
int printErrorConsole(lua_State* L)
{
    return Priv::printParameters(L, cerr);
}
//------------------------------------------------------------------------------
// Paths
/** Get the path of the common data dir (data/common) */
std::string GetCommonDataPath()
{
    return g_global->confManager.GetCommonDataPath();
}
//------------------------------------------------------------------------------
/** Get the path of the mode dir (data/modes/[modeName]) */
std::string GetModePath()
{
    return g_global->confManager.GetModePath();
}
//------------------------------------------------------------------------------
/** Get the avaible video modes */
luabind::object GetAvaibleVideoModes()
{
    luabind::object table = luabind::newtable(g_global->luaState);
    //table[1] = luabind::object();
    unsigned int previousBbp = 0;
    table[1] = Vector2D(-1, -1);
    for(size_t i = 0; i < sf::VideoMode::GetModesCount(); i++)
    {
        const sf::VideoMode mode = sf::VideoMode::GetMode(i);

        // We exclude the modes with a lower BBP
        if(i != 0 && mode.BitsPerPixel < previousBbp)
        {
            break;
        }
        previousBbp = mode.BitsPerPixel;

        // We add the mode to the table
        table[i + 2] = Vector2D(mode.Width, mode.Height);
    }
    return table;
}
//------------------------------------------------------------------------------
/** Get the arguments given to bottle2d when executed (return a table of string). */
luabind::object GetCmdArguments()
{
    luabind::object table = luabind::newtable(g_global->luaState);

    for(size_t i = 0; i < g_global->cmdArgTbl.size(); i++)
    {
        table[i + 1] = g_global->cmdArgTbl[i];
    }
    return table;
}
//------------------------------------------------------------------------------
/** Get the arguments given to bottle2d when executed (return a table of std::string). */
std::vector<std::string> GetStdCmdArguments()
{
    return g_global->cmdArgTbl;
}
//------------------------------------------------------------------------------
namespace Priv
{
    //------------------------------------------------------------------------------
    void CreateRenderWindow()
    {
        if(g_global->renderWindow != NULL)
        {
            delete g_global->renderWindow;
        }

        g_global->renderWindow = g_global->confManager.CreateRenderWindow();
    }
    //------------------------------------------------------------------------------
    int printParameters(lua_State* L, std::ostream &stream)
    {
        int argCount = lua_gettop(L);

        for(int i = 1; i <= argCount; i++)
        {
            luabind::object obj(luabind::from_stack(L, i));

            switch(luabind::type(obj))
            {
                case LUA_TSTRING:
                    stream << luabind::object_cast<std::string>(obj);
                    break;
                case LUA_TNUMBER:
                    stream << luabind::object_cast<double>(obj);
                    break;
                case LUA_TBOOLEAN:
                    stream << boolalpha << luabind::object_cast<bool>(obj);
                    break;
                case LUA_TNIL:
                    stream << "#Nil#";
                    break;
                default:
                    stream << "#Unknown type '" << luabind::type(obj) << "'#";
                    break;
            }
        }

        stream << endl;

        return 0;
    }
    //------------------------------------------------------------------------------
    void CheckEngine() throw()
    {
        if(!g_globalOk)
        {
            MY_THROW("CheckEngine : Using a non ready engine !");
        }
    }
    //------------------------------------------------------------------------------
    void LoadLuaState()
    {
        if(g_global->luaState != NULL)
        {
            lua_close(g_global->luaState);
            g_global->luaState = NULL;
        }

        // Create a new lua state
        g_global->luaState = lua_open();
        // Connect LuaBind to this lua state
        luabind::open(g_global->luaState);

        /*luaopen_base(g_global->luaState);
        luaopen_table(g_global->luaState);
        luaopen_string(g_global->luaState);
        luaopen_math(g_global->luaState);
        // FIXME : Keep io ?
        luaopen_io(g_global->luaState);*/

        luaL_openlibs(g_global->luaState);

        // Panic
        lua_atpanic(g_global->luaState, Priv::OnLuaPanic);

        // Expose
        ExposeLua(g_global->luaState);

        // Print functions
        lua_register(g_global->luaState, "print", printConsole);
        lua_register(g_global->luaState, "printConsole", printConsole);
        lua_register(g_global->luaState, "printErrorConsole", printErrorConsole);
    }
    //------------------------------------------------------------------------------
    int OnLuaPanic(lua_State *state)
    {
        MY_THROW("Lua Panic !!!");
    }
    //------------------------------------------------------------------------------
    void OnHotkeyQuit(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
    {
        const sf::Input &input = g_global->renderWindow->GetInput();
        if(input.IsKeyDown(HOTKEY_KEY)) // LControl should be pressed too
        {
            cout << "Quit hot key received" << endl;
            Quit();
        }
    }
    //------------------------------------------------------------------------------
    void OnHotkeySwitchFullscreen(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
    {
        const sf::Input &input = g_global->renderWindow->GetInput();
        if(input.IsKeyDown(HOTKEY_KEY)) // LControl should be pressed too
        {
			cout << "Fullscreen / Windowed hot key received" << endl;
			ConfigNode node = GetConfigManager().GetGlobalConfigNode().AccessNode("graphicConf");
			node.SetValue("isFullscreen", !node.GetNumber("isFullscreen"));
			RequestRecreateRenderWindow();
        }
    }
    //------------------------------------------------------------------------------
    void OnHotkeyShowConsole(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
    {
        const sf::Input &input = g_global->renderWindow->GetInput();
        if(input.IsKeyDown(HOTKEY_KEY)) // LControl should be pressed too
        {
			cout << "Console hot key received" << endl;
			GetConsole().Show(! GetConsole().IsVisible());
        }
    }
    //------------------------------------------------------------------------------
    void OnHotkeyMenu(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
    {
        const sf::Input &input = g_global->renderWindow->GetInput();
        if(input.IsKeyDown(HOTKEY_KEY)) // LControl should be pressed too
        {
			cout << "Main menu hot key received" << endl;
			LoadMode("Menu");
        }
    }
    //------------------------------------------------------------------------------
    void OnHotkeyShowPhysicObj(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
    {
        const sf::Input &input = g_global->renderWindow->GetInput();
        if(input.IsKeyDown(HOTKEY_KEY)) // LControl should be pressed too
        {
			cout << "Show physic objects hot key received" << endl;
			SetShowPhysicObjects(!IsShowPhysicObjects());
        }
    }
    //------------------------------------------------------------------------------
    /** Add hotkeys events to a screen */
    void AddHotKeysEvents(Screen *screen)
    {
        ButtonEventHandler *evt = static_cast<ButtonEventHandler*>(screen->RegisterButtonEvent(luabind::object(), "F1", true).GetPtr());
        evt->SetEventFunction(&Priv::OnHotkeyShowConsole);

        evt = static_cast<ButtonEventHandler*>(screen->RegisterButtonEvent(luabind::object(), "F2", true).GetPtr());
        evt->SetEventFunction(&Priv::OnHotkeySwitchFullscreen);

        evt = static_cast<ButtonEventHandler*>(screen->RegisterButtonEvent(luabind::object(), "F3", true).GetPtr());
        evt->SetEventFunction(&Priv::OnHotkeyMenu);

        evt = static_cast<ButtonEventHandler*>(screen->RegisterButtonEvent(luabind::object(), "F4", true).GetPtr());
        evt->SetEventFunction(&Priv::OnHotkeyQuit);

        evt = static_cast<ButtonEventHandler*>(screen->RegisterButtonEvent(luabind::object(), "F5", true).GetPtr());
        evt->SetEventFunction(&Priv::OnHotkeyShowPhysicObj);
    }
    //------------------------------------------------------------------------------
    void DoLoadMode()
    {
        string tmpModeName = g_global->modeToLoad; // In case loading the mode fail, we need modeToLoad to be empty.

        // We clear the current mode and reset lua state
        Clear();

        // We load the new mode
        g_global->confManager.LoadMode(tmpModeName);

        // Window name
        //g_global->renderWindow->SetTitle(string(PACKAGE_NAME) + " - " + GetModeInfo().fullName);

        // Lua lunch
        Global::LuaRunScript("main.lua");

        // Create a frame to prevente huge frame time and physic pb
        g_global->renderWindow->Display();
        g_global->renderWindow->Display();
    }
    //------------------------------------------------------------------------------
    /** Get the child tree (for debuging purpose) */
    void GetChildTreeRecursive(std::string &text, int level, Group *group)
    {
        string margin;
        margin.append(level * 3, ' ');

        int count = group->GetDrawableChildCount();
        if(count != 0)
        {
            text += margin + "Drawable:\n";
        }

        for(int i = 0; i < count; i++)
        {
            Drawable *obj = group->GetDrawableChildUncast(i);

            text += margin + "|- " + obj->GetName() + " (" + GetTypeName(obj->GetType()) + ")\n";

            // Group
            if(obj->GetType() == OBJ_TYPE_DRAWABLE_GROUP || obj->GetType() == OBJ_TYPE_DRAWABLE_SCREEN)
            {
                GetChildTreeRecursive(text, level + 1, static_cast<Group*>(obj));
            }
        }

        count = group->GetPhysicChildCount();
        if(count != 0)
        {
            text += margin + "Physic:\n";
        }

        for(int i = 0; i < count; i++)
        {
            PhysicObject *obj = group->GetPhysicChildUncast(i);
            text += margin + "|- " + obj->GetName() + " (" + GetTypeName(obj->GetType()) + ")\n";
        }
    }
    //------------------------------------------------------------------------------

} // Namespace priv
//------------------------------------------------------------------------------
/** Get the child tree (for debuging purpose) */
std::string GetChildTree()
{
    if(g_global->currentScreen.IsNull())
    {
        MY_THROW("GetChildTree() : Current screen is null !")
    }

    std::string text;
    text += g_global->currentScreen->GetName() + " (" + GetTypeName(g_global->currentScreen->GetType()) + ")\n";
    Priv::GetChildTreeRecursive(text, 0, g_global->currentScreen);

    return text;
}
//------------------------------------------------------------------------------
float ConvertRatioCosAccelerationDeceleration(float ratio)
{
    if(ratio == 1.f || ratio == 0.f)
    {
        return ratio;
    }

    return (- cos(ratio * 3.14) + 1) / 2;
}
//------------------------------------------------------------------------------
float ConvertRatioAcceleration(float ratio, float coefficient)
{
    if(ratio == 1.f || ratio == 0.f)
    {
        return ratio;
    }

    coefficient = abs(coefficient);

    ratio = 1.f - ratio;
    ratio = - (exp(- ratio * coefficient) - exp(-coefficient) * ratio) + 1.f;
    ratio = 1.f - ratio;
    return ratio;
}
//------------------------------------------------------------------------------
float ConvertRatioDeceleration(float ratio, float coefficient)
{
    if(ratio == 1.f || ratio == 0.f)
    {
        return ratio;
    }

    coefficient = abs(coefficient);
    return - (exp(- ratio * coefficient) - exp(-coefficient) * ratio) + 1.f;
}
//------------------------------------------------------------------------------
Vector2D GetMousePosition()
{
    sf::RenderWindow &App = *g_global->renderWindow;
    const sf::Input& Input = App.GetInput();

    return Vector2D(App.ConvertCoords(Input.GetMouseX(), Input.GetMouseY()));
}
//------------------------------------------------------------------------------
ConfigNode GetModeConf()
{
    Priv::CheckEngine();

    return g_global->confManager.GetRootConfigNode();
}
//------------------------------------------------------------------------------
ModeInfo GetModeInfo()
{
    Priv::CheckEngine();

    return g_global->confManager.GetModeInfo(g_global->confManager.GetCurrentModeId());
}
//------------------------------------------------------------------------------
void LoadMode(const std::string &idName)
{
    g_global->modeToLoad = idName;
}
//------------------------------------------------------------------------------
void Clear()
{
    // Stop global sounds
    g_global->globalSoundManager->ClearAllSounds();

    // No mode to load (won't try to load a mode in safe mode)
    g_global->modeToLoad = "";

    // No current screen
    g_global->currentScreen = NULL;

    // Unique frame id
    g_global->uniqueFrameId = 0;

    // On supprime les screen du précédent mode
    for(std::vector<Screen*>::iterator it = g_global->screenTbl.begin(); it != g_global->screenTbl.end(); it++ )
    {
        delete *it;
    }
    g_global->screenTbl.clear();

    Priv::LoadLuaState(); // reset the lua state
}
//------------------------------------------------------------------------------
void ForceFrame()
{
    Priv::CheckEngine();

    if(g_global->renderWindow == NULL)
    {
        MY_THROW("ForceFrame : g_global->RenderWindow is null !");
    }

    if(g_global->currentScreen.IsNull())
    {
        MY_THROW("ForceFrame : g_global->currentScreen is null !");
    }

    if(g_global->currentScreen.IsValid())
    {
        MY_THROW("ForceFrame : g_global->currentScreen is was destroyed !");
    }

    g_global->renderWindow->Clear(g_global->currentScreen->GetBackgroundColor());
    g_global->currentScreen->Render(*g_global->renderWindow);
    g_global->console->Render(*g_global->renderWindow);

    // Unique frame id
    g_global->uniqueFrameId++;

    // Display window on screen
    g_global->renderWindow->Display();
}
//------------------------------------------------------------------------------
/** Return true if the key \\a keyName is pressed, false otherwise */
bool IsKeyPressed(const std::string &keyName)
{
    return En::IsKeyPressed(keyName, g_global->renderWindow->GetInput());
}
//------------------------------------------------------------------------------
/** Return the real size of the window (resolution). This function should not be used unless you want to know the real window ratio.\\n
  * See ScreenConfig.
  */
Vector2D GetRealWindowSize()
{
    return Vector2D(g_global->renderWindow->GetWidth(), g_global->renderWindow->GetHeight());
}
//------------------------------------------------------------------------------
/** Set if the group centers and the physic objects should be drawn (helps for debuging). */
void SetShowPhysicObjects(bool isVisible)
{
    g_global->drawPhysicObjects = isVisible;
}
//------------------------------------------------------------------------------
/** Return true if the group centers and the physic objects should be drawn, else otherwise. */
bool IsShowPhysicObjects()
{
    return g_global->drawPhysicObjects;
}
//------------------------------------------------------------------------------
/** Translation function (do nothing) */
std::string _(const std::string &str)
{
    return gettext(str.c_str());
}
//------------------------------------------------------------------------------
/** Return the type name (string) of the given type id */
std::string GetTypeName(int typeId)
{
    switch(typeId)
    {
        #define X(typeName, typeId) \
            case typeId: \
                return #typeName; \
                break;
        X_OBJ_LIST
        #undef X
        default:
            return "Unknown";
            break;
    }
}
//------------------------------------------------------------------------------
/** Get the global sound manager. A sound launched into the global SoundManager will not be paused when the screen is changed. */
SoundManager *GetGlobalSoundManager()
{
    return g_global->globalSoundManager;
}
//------------------------------------------------------------------------------
/** Get the current running screen. */
ProtectedPtr<Screen> GetCurrentScreen()
{
    if(g_global->currentScreen.IsValid())
    {
        return g_global->currentScreen;
    }
    else if(g_global->currentScreen.IsNull())
    {
        MY_THROW("ProtectedPtr<Screen> GetCurrentScreen() : No screen set !");
    }
    else
    {
        MY_THROW("ProtectedPtr<Screen> GetCurrentScreen() : The current screen was deleted !");
    }
}
//------------------------------------------------------------------------------
/** Get an asssociative array of the Screen with their name as key. */
luabind::object GetScreenList()
{
    luabind::object table = luabind::newtable(g_global->luaState);
    for(std::vector<Screen*>::iterator it = g_global->screenTbl.begin(); it != g_global->screenTbl.end(); it++)
    {
        table[(*it)->GetName()] = ProtectedPtr<Screen>(*it);
    }

    return table;
}
//------------------------------------------------------------------------------
} // namespace Global
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
} // namespace En
